#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Grec <jgrec@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_SoftwareIdentityResource`` provider.
"""

import os
import pywbem
import time
import shutil
import subprocess
import unittest

from lmi.test.lmibase import enable_lmi_exceptions
import repository
import swbase
import util

ENABLED_DEFAULT_NOT_APPLICABLE = 5
ENABLED_STATE_DISABLED = 3
ENABLED_STATE_ENABLED = 2
EXTENDED_RESOURCE_TYPE_RPM = 3
HEALTH_STATE_MAJOR_FAILURE = 20
HEALTH_STATE_OK = 5
INFO_FORMAT_URL = 200
OPERATIONAL_STATUS_ERROR = 6
OPERATIONAL_STATUS_OK = 2
REQUESTED_STATE_ENABLED = 2
REQUESTED_STATE_DISABLED = 3
REQUEST_STATE_CHANGE_SUCCESSFUL = 0
REQUEST_STATE_CHANGE_ERROR = 2
RESOURCE_TYPE_OTHER = 1
PRIMARY_STATUS_ERROR = 3
PRIMARY_STATUS_OK = 1
TRANSITIONING_TO_STATE_NOT_APPLICABLE = 12

DUMMY_TEST_REPO = 'lmi-test'

class TestSoftwareIdentityResource(swbase.SwTestCase):
    """
    Basic cim operations test on ``LMI_SoftwareIdentityResource``.
    """

    CLASS_NAME = "LMI_SoftwareIdentityResource"
    KEYS = ("CreationClassName", "Name", "SystemCreationClassName",
            "SystemName")

    def make_op(self, repo):
        """
        :returns: Object path of ``LMI_SoftwareIdentityResource``
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        if isinstance(repo, repository.Repository):
            repo = repo.repoid
        return self.cim_class.new_instance_name({
            "Name" : repo,
            "SystemName" : self.SYSTEM_NAME,
            "SystemCreationClassName" : self.system_cs_name,
            "CreationClassName" : self.CLASS_NAME
        })

    def _check_repo_instance(self, repo, inst):
        """
        Checks instance properties of repository.
        """
        objpath = self.make_op(repo)
        self.assertCIMNameEqual(objpath, inst.path)
        for key in self.KEYS:
            if key == 'SystemName':
                self.assertIn(inst.SystemName, (self.SYSTEM_NAME, 'localhost'))
                self.assertIn(inst.path.SystemName, (self.SYSTEM_NAME, 'localhost'))
            else:
                self.assertEqual(getattr(inst, key), getattr(inst.path, key))
        self.assertEqual(inst.Name, repo.repoid)

        self.assertIsInstance(inst.AccessContext, pywbem.Uint16)
        access_info = inst.AccessInfo
        if access_info is not None:
            access_info = access_info.rstrip('/')
        if access_info is not None:
            self.assertIsInstance(access_info, basestring)
        if repo.mirror_list:
            self.assertEqual(access_info, repo.mirror_list,
                    "AccessInfo does not match mirror_list for repo %s" %
                    repo.repoid)
        elif repo.metalink:
            self.assertEqual(access_info, repo.metalink,
                    "AccessInfo does not match metalink for repo %s" %
                    repo.repoid)
        elif access_info is not None:
            self.assertIn(access_info, {u.rstrip('/') for u in repo.base_urls},
                "AccessInfo missing in base_urls for repo %s" % repo.repoid)

        self.assertIsInstance(inst.AvailableRequestedStates, list)
        # Unfortunately, yum-config-manager shortens lines on its output
        # therefor we can not check very long names.
        # 61 characters in version yum-utils-1.1.31-19.fc21
        if len(repo.name) > 60:
            self.assertTrue(inst.Caption.startswith(repo.name))
        else:
            self.assertEqual(inst.Caption, repo.name)
        self.assertIsInstance(inst.Cost, pywbem.Sint32)
        self.assertIsInstance(inst.Description, basestring)
        self.assertEqual(inst.ElementName, repo.repoid)
        self.assertEqual(inst.EnabledDefault, ENABLED_DEFAULT_NOT_APPLICABLE)
        self.assertEqual(
                       ENABLED_STATE_ENABLED
                    if repo.status else ENABLED_STATE_DISABLED,
                inst.EnabledState,
                "EnabledState does not match for repo %s" % repo.repoid)
        self.assertEqual(inst.ExtendedResourceType, EXTENDED_RESOURCE_TYPE_RPM)
        if repo.revision is not None:
            self.assertTrue(isinstance(inst.Generation, (int, long)),
                    'Generation must be integer for repo %s' % repo.repoid)
        if repo.status:
            self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
            self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
            self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)
        else:
            # repo may not be enabled, but still it can be ready
            # (which is not documented), so let's check both posibilities
            # TODO: health state should be OK for disabled repository
            self.assertIn(inst.HealthState,
                    [HEALTH_STATE_OK, HEALTH_STATE_MAJOR_FAILURE])
            self.assertIn(inst.OperationalStatus,
                    [[OPERATIONAL_STATUS_OK], [OPERATIONAL_STATUS_ERROR]])
            self.assertIn(inst.PrimaryStatus, [PRIMARY_STATUS_OK, PRIMARY_STATUS_ERROR])
        self.assertIsInstance(inst.GPGCheck, bool)
        self.assertEqual(inst.InfoFormat, INFO_FORMAT_URL)
        self.assertEqual("LMI:LMI_SoftwareIdentityResource:"+repo.repoid,
                inst.InstanceID)
        self.assertEqual(inst.MirrorList, repo.mirror_list,
                'MirrorList must match for repo %s' % repo.repoid)
        self.assertIsInstance(inst.OtherAccessContext, basestring)
        self.assertIsInstance(inst.RepoGPGCheck, bool)
        self.assertEqual(
                   ENABLED_STATE_ENABLED
                if repo.status else ENABLED_STATE_DISABLED,
                inst.RequestedState)
        self.assertEqual(inst.ResourceType, RESOURCE_TYPE_OTHER)
        self.assertIsInstance(inst.StatusDescriptions, list)
        self.assertEqual(1, len(inst.StatusDescriptions))
        if repo.last_updated is not None:
            time_stamp = repo.last_updated.replace(
                    microsecond=0, tzinfo=pywbem.cim_types.MinutesFromUTC(0))
            self.assertGreaterEqual(inst.TimeOfLastUpdate.datetime, time_stamp)
        self.assertEqual(TRANSITIONING_TO_STATE_NOT_APPLICABLE,
                inst.TransitioningToState)

    @swbase.test_with_repos(stable=True, updates=False)
    def test_get_instance(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityResource``.
        """
        stablerepo = self.get_repo('stable')
        self.assertTrue(stablerepo.status, "stable repository is enabled")
        objpath = self.make_op(stablerepo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                "GetInstance() call on stable repository is successful"
                ' with path: "%s"' % str(objpath))
        self._check_repo_instance(stablerepo, inst)
        self.assertEqual(set(self.KEYS), set(inst.path.key_properties()))

        updaterepo = self.get_repo('updates')
        objpath = self.make_op(updaterepo)
        self.assertFalse(updaterepo.status, "updates repository is disabled")
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on updates repository is successful'
                ' with path "%s"' % str(objpath))
        self._check_repo_instance(updaterepo, inst)
        self.assertEqual(set(self.KEYS), set(inst.path.key_properties()))

    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames()`` call on ``LMI_SoftwareIdentityResource``.
        """
        all_repo_names = set(self.repodb.keys()).union(
                set(self.other_repos.keys()))
        inames = self.cim_class.instance_names()
        for iname in inames:
            self.assertEqual(set(iname.key_properties()), set(self.KEYS))
            self.assertIn(iname.Name, all_repo_names)
            all_repo_names.remove(iname.Name)
            self.assertEqual(iname.SystemCreationClassName, self.system_cs_name)
            self.assertEqual(iname.CreationClassName, self.CLASS_NAME)
        self.assertEqual(len(all_repo_names), 0)

    def test_enum_instances(self):
        """
        Test ``EnumInstances()`` call on ``LMI_SoftwareIdentityResource``.
        """
        all_repo_names = set(self.repodb.keys()).union(
                set(self.other_repos.keys()))
        insts = self.cim_class.instances()
        for inst in insts:
            self.assertEqual(set(inst.path.key_properties()), set(self.KEYS))
            self.assertIn(inst.Name, all_repo_names)
            all_repo_names.remove(inst.Name)
            if inst.Name in self.repodb:
                repo = self.repodb[inst.Name]
            else:
                repo = self.other_repos[inst.Name]
            self._check_repo_instance(repo, inst)
        self.assertEqual(len(all_repo_names), 0)

    @swbase.test_with_repos('updates')
    def test_disable_repo(self):
        """
        Test whether ``LMI_SoftwareIdentityResource`` provider recognizes
        that repository has been disabled.
        """
        repo = self.get_repo('updates')
        self.assertTrue(repo.status)
        objpath = self.make_op(repo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on updates repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_ENABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

        repository.set_repos_enabled(repo, False)
        repo.refresh()
        self.assertFalse(repo.status)
        time.sleep(0.1)
        inst.refresh()
        self.assertNotEqual(inst, None,
                'GetInstance() call on updates repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_DISABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)


    @swbase.test_with_repos(updates=False)
    def test_enable_repo(self):
        """
        Test whether ``LMI_SoftwareIdentityResource`` provider recognizes that
        repository has been enabled.
        """
        repo = self.get_repo('updates')
        self.assertFalse(repo.status)
        objpath = self.make_op(repo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on updates repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_DISABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

        repository.set_repos_enabled(repo, True)
        repo.refresh()
        self.assertTrue(repo.status)
        inst.refresh()
        self.assertNotEqual(inst, None,
                'GetInstance() call on updates repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_ENABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

    @swbase.test_with_repos('stable')
    def test_disable_enabled_repo(self):
        """
        Test ``RequestStateChange()`` method invocation on
        ``LMI_SoftwareIdentityResource`` by requesting disablement of
        repository.
        """
        repo = self.get_repo('stable')
        self.assertTrue(repo.status)
        objpath = self.make_op(repo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on stable repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_ENABLED)

        # try to disable it with method invocation
        (rval, oparms, _) = inst.RequestStateChange(
                RequestedState=REQUESTED_STATE_DISABLED)
        self.assertEqual(rval, REQUEST_STATE_CHANGE_SUCCESSFUL)
        self.assertEqual(len(oparms), 0)
        repo.refresh()
        self.assertFalse(repo.status)
        inst.refresh()
        self.assertEqual(inst.EnabledState, ENABLED_STATE_DISABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

    @swbase.test_with_repos(stable=False)
    def test_enable_disabled_repo(self):
        """
        Test ``RequestStateChange()`` method invocation on
        ``LMI_SoftwareIdentityResource`` by requesting enablement of
        repository.
        """
        repo = self.get_repo('stable')
        self.assertFalse(repo.status)
        objpath = self.make_op(repo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on stable repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_DISABLED)

        # try to enable it with method invocation
        (rval, oparms, _) = inst.RequestStateChange(
                RequestedState=REQUESTED_STATE_ENABLED)
        self.assertEqual(rval, REQUEST_STATE_CHANGE_SUCCESSFUL)
        self.assertEqual(len(oparms), 0)
        repo.refresh()
        self.assertTrue(repo.status)
        inst.refresh()
        self.assertEqual(inst.EnabledState, ENABLED_STATE_ENABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

    @swbase.test_with_repos('stable')
    def test_enable_enabled_repo(self):
        """
        Test ``RequestStateChange()`` method invocation on
        ``LMI_SoftwareIdentityResource`` by requesting enablement of
        enabled repository.
        """
        repo = self.get_repo('stable')
        self.assertTrue(repo.status)
        objpath = self.make_op(repo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on stable repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_ENABLED)

        # try to enable it with method invocation
        (rval, oparms, _) = inst.RequestStateChange(
                RequestedState=REQUESTED_STATE_ENABLED)
        self.assertEqual(rval, REQUEST_STATE_CHANGE_SUCCESSFUL)
        self.assertEqual(len(oparms), 0)
        repo.refresh()
        self.assertTrue(repo.status)
        inst.refresh()
        self.assertEqual(inst.EnabledState, ENABLED_STATE_ENABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

    @swbase.test_with_repos(stable=False)
    def test_disable_disabled_repo(self):
        """
        Test ``RequestStateChange()`` method invocation on
        ``LMI_SoftwareIdentityResource`` by requesting disablement of
        disabled repository.
        """
        repo = self.get_repo('stable')
        self.assertFalse(repo.status)
        objpath = self.make_op(repo)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None,
                'GetInstance() call on stable repository is successful'
                ' with path "%s"' % str(objpath))
        self.assertEqual(inst.EnabledState, ENABLED_STATE_DISABLED)

        # try to enable it with method invocation
        (rval, oparms, _) = inst.RequestStateChange(
                RequestedState=REQUESTED_STATE_DISABLED)
        self.assertEqual(rval, REQUEST_STATE_CHANGE_SUCCESSFUL)
        self.assertEqual(len(oparms), 0)
        repo.refresh()
        self.assertFalse(repo.status)
        inst.refresh()
        self.assertEqual(inst.EnabledState, ENABLED_STATE_DISABLED)
        self.assertEqual(inst.HealthState, HEALTH_STATE_OK)
        self.assertEqual(inst.OperationalStatus, [OPERATIONAL_STATUS_OK])
        self.assertEqual(inst.PrimaryStatus, PRIMARY_STATUS_OK)

    @enable_lmi_exceptions
    def test_disable_unexisting_repository(self):
        """
        Test disable unexisting repository.

        Enable DUMMY_TEST_REPO repository, get its instance, delete it
        from /etc/yum.repos.d and try to disable it by OpenLMI.
        Cases:
            OpenLMI won't disable repository - returns error in LMIReturnValue
        """
        repo_file_path = os.path.join(self.yum_repos_dir,
                DUMMY_TEST_REPO + ".repo")
        shutil.copy2(
                "%s/%s.repo" % (
                    os.path.dirname(swbase.__file__),
                    DUMMY_TEST_REPO),
                repo_file_path)
        try:
            repository.set_repos_enabled(DUMMY_TEST_REPO, True)
            subprocess.call(["/usr/bin/yum-config-manager",
                "--enable", DUMMY_TEST_REPO],
                stdout=util.DEV_NULL, stderr=util.DEV_NULL)
            repo = self.cim_class.first_instance_name(
                    {"Name": DUMMY_TEST_REPO}).to_instance()
            self.assertNotEqual(repo, None, "dummy repository exists")

            os.remove(repo_file_path)

            self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND,
                    repo.RequestStateChange,
                    RequestedState=REQUESTED_STATE_DISABLED)
        finally:
            if os.path.exists(repo_file_path):
                os.remove(repo_file_path)

    @enable_lmi_exceptions
    def test_enable_unexisting_repository(self):
        """
        Test enable deleted repository.

        Get DUMMY_TEST_REPO's instance, delete it from /etc/yum.repos.d
        and try to enable it by OpenLMI.
        Cases:
            OpenLMI won't enable repository - returns error in LMIReturnValue
        """
        repo_file_path = os.path.join(self.yum_repos_dir,
                DUMMY_TEST_REPO + ".repo")
        shutil.copy2(
                "%s/%s.repo" % (
                    os.path.dirname(swbase.__file__),
                    DUMMY_TEST_REPO),
                repo_file_path)
        try:

            repo = self.cim_class.first_instance_name(
                    {"Name": DUMMY_TEST_REPO}).to_instance()

            os.remove(repo_file_path)

            self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND,
                    repo.RequestStateChange,
                    RequestedState=REQUESTED_STATE_ENABLED)
        finally:
            if os.path.exists(repo_file_path):
                os.remove(repo_file_path)

def suite():
    """ For unittest loaders. """
    return unittest.TestLoader().loadTestsFromTestCase(
            TestSoftwareIdentityResource)

if __name__ == '__main__':
    unittest.main()



# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Since yum API functions should not be called with different thread_ids
repeatedly in the same program. It's neccessary, to make these calls
in single thread. But the provider needs to be able to clean up itself,
when its not needed. That's why the yum API needs to be accessed from
separated process, that is created and terminated when needed.

This package contains all the bowels of this separate process together
with its management and communication facilities.

YumDB is a context manager supposed to be used by any provider as the
only accessor to yum api.
"""

from functools import wraps
import inspect
import threading

from lmi.base import singletonmixin
from lmi.providers import cmpi_logging
from lmi.software.yumdb import jobs
from lmi.software.yumdb.jobmanager import JobManager
from lmi.software.yumdb.packagecheck import PackageCheck
from lmi.software.yumdb.packagecheck import PackageFile
from lmi.software.yumdb.packageinfo import PackageInfo
from lmi.software.yumdb.repository import Repository

LOG = cmpi_logging.get_logger(__name__)

# *****************************************************************************
# Utilities
# *****************************************************************************
def _make_async_job(jobcls, *args, **kwargs):
    """Creates asynchronous job, filling it wih some metadata."""
    if not issubclass(jobcls, jobs.YumAsyncJob):
        raise TypeError("jobcls must be a subclass of YumAsyncJob")
    job = jobcls(*args, **kwargs)
    if job.metadata is None:
        job.metadata = {}
    job.metadata['name'] = \
            type(job).__name__[len('Yum'):] + ('-%d' % job.jobid)
    return job

# *****************************************************************************
# Decorators
# *****************************************************************************
def job_request(async=False):
    """
    Decorator factory for job entry points. They are YumDB methods.
    All of them must return either job objects or jobid for asynchronous calls.
    Job objects are processed by this decorator for caller to obtain only the
    information he needs.

    It wrapps them with logger wrapper and in case of asynchronous jobs,
    it returns just the jobid.
    """
    def _decorator(method):
        """
        Decorator that just logs the method's call and returns job's result.
        """
        logged = cmpi_logging.trace_method(method, 2)
        @wraps(method)
        def _new_func(self, *args, **kwargs):
            """Wrapper for YumDB's method."""
            return logged(self, *args, **kwargs).result_data
        return _new_func

    def _decorator_async(method):
        """
        Decorator for methods accepting async argument. In case of async=True,
        the method returns jobid. Job's result is returned otherwise.
        """
        logged = cmpi_logging.trace_method(method, 2)
        @wraps(method)
        def _new_func(self, *args, **kwargs):
            """Wrapper for YumDB's method."""
            callargs = inspect.getcallargs(method, self, *args, **kwargs)
            result = logged(self, *args, **kwargs)
            if callargs.get('async', False):
                return result
            else:
                return result.result_data
        return _new_func

    return _decorator_async if async else _decorator

class YumDB(singletonmixin.Singleton):
    """
    Context manager for accessing yum/rpm database.
    All requests are bundled into jobs -- instances of jobs.YumJob and
    sent to YumWorker for processing.

    YumWorker is a separate process handling all calls to yum api.
    Communication is done via queues (uplink and downlink).
    Uplink is used to send jobs to YumWorker and downlink for obtaining
    results.

    This is implemented in thread safe manner.

    It should be used as a context manager in case, we want to process
    multiple jobs in single transaction. The example of usage:
        with YumDB.getInstance() as ydb:
            pkgs = ydb.filter_packages(...)
            for pkg in pkgs:
                ydb.install_package(pkg)
            ...
    Yum database stays locked in whole block of code under with statement.
    """

    # this is to inform Singleton, that __init__ should be called only once
    ignoreSubsequent = True

    @cmpi_logging.trace_method
    def __init__(self):   #pylint: disable=W0231
        """
        All arguments are passed to yum.YumBase constructor.
        """
        self._access_lock = threading.RLock()
        self._jobmgr = None
        LOG().trace_info('YumDB initialized')

    # *************************************************************************
    # Private methods
    # *************************************************************************
    @property
    def jobmgr(self):
        with self._access_lock:
            if self._jobmgr is None:
                self._jobmgr = JobManager()
                self._jobmgr.start()
            return self._jobmgr

    def _do_job(self, job):
        """
        Sends the job to YumWorker process and waits for reply.
        If reply is a tuple, there was an error, while job processing.
        Incoming exception is in format:
          (exception_type, exception_value, formated_traceback_as_string)
        @return reply
        """
        LOG().trace_verbose("doing %s", job)
        reply = self.jobmgr.process(job)
        jobs.log_reply_error(job, reply)
        LOG().trace_verbose("job %s done", job.jobid)
        return reply

    # *************************************************************************
    # Special methods
    # *************************************************************************
    @cmpi_logging.trace_method
    def __enter__(self):
        """ Start a new session. """
        with self._access_lock:
            self.jobmgr.begin_session()
        return self

    @cmpi_logging.trace_method
    def __exit__(self, exc_type, exc_value, traceback):
        """ End active session. """
        with self._access_lock:
            self.jobmgr.end_session()

    # *************************************************************************
    # Public methods
    # *************************************************************************
    @cmpi_logging.trace_method
    def clean_up(self):
        """
        Shut down job manager and all the other threads and processes it has
        created.
        """
        with self._access_lock:
            if self._jobmgr is not None:
                self._do_job(jobs.YumShutDown())
                self._jobmgr.join()
                self._jobmgr = None
            else:
                LOG().warn("clean_up called with JobManager not initialized")

    # *************************************************************************
    # Jobs with simple results
    # *************************************************************************
    @job_request()
    def get_package_list(self, kind,
            allow_duplicates=False,
            sort=False,
            include_repos=None,
            exclude_repos=None):
        """
        @param kind is one of: jobs.YumGetPackageList.SUPPORTED_KINDS
        @param allow_duplicates says, whether to list all found versions
        of single package
        @return [pkg1, pkg2, ...], pkgi is instance of yumdb.PackageInfo
        """
        return self._do_job(jobs.YumGetPackageList(
            kind, allow_duplicates=allow_duplicates, sort=sort,
            include_repos=include_repos, exclude_repos=exclude_repos))

    @job_request()
    def filter_packages(self, kind,
            allow_duplicates=False,
            exact_match=True,
            sort=False,
            include_repos=None,
            exclude_repos=None,
            **filters):
        """
        Similar to get_package_list(), but applies filter on packages.
        @see yumdb.jobs.YumFilterPackages job for supported filter keys
        """
        return self._do_job(jobs.YumFilterPackages(
            kind, allow_duplicates=allow_duplicates, exact_match=exact_match,
            sort=sort, include_repos=include_repos, exclude_repos=exclude_repos,
            **filters))

    @job_request()
    def get_repository_list(self, kind):
        """
        @param kind is one of: jobs.YumGetRepositoryList.SUPPORTED_KINDS
        @param allow_duplicates says, whether to list all found versions
        of single package
        @return [pkg1, pkg2, ...], pkgi is instance of yumdb.Repository
        """
        return self._do_job(jobs.YumGetRepositoryList(kind))

    @job_request()
    def filter_repositories(self, kind, **filters):
        """
        Similar to get_repository_list(), but applies filter on packages.
        @see yumdb.jobs.YumFilterRepositories job for supported filter keys
        """
        return self._do_job(jobs.YumFilterRepositories(kind, **filters))

    @job_request()
    def set_repository_enabled(self, repoid, enable):
        """
        Enable or disable repository.
        @param enable is a boolean
        """
        return self._do_job(jobs.YumSetRepositoryEnabled(repoid, enable))

    # *************************************************************************
    # Asynchronous jobs
    # *************************************************************************
    @job_request(async=True)
    def install_package(self, pkg, async=False, force=False, **metadata):
        """
        Install package.
        @param pkg is an instance of PackageInfo obtained with
        get_package_list() or filter_packages() or a valid nevra as string.
        Package must not be installed if force is False.
        """
        return self._do_job(_make_async_job(jobs.YumInstallPackage,
            pkg, force=force, async=async, metadata=metadata))

    @job_request(async=True)
    def remove_package(self, pkg, async=False, **metadata):
        """
        @param pkg is an instance of PackageInfo obtained with
        get_package_list() or filter_packages(), which must be installed
        """
        return self._do_job(_make_async_job(jobs.YumRemovePackage,
            pkg, async=async, metadata=metadata))

    @job_request(async=True)
    def update_to_package(self, desired_pkg, async=False, **metadata):
        """
        @param desired_pkg is an instance of PackageInfo,
        which must be available
        """
        return self._do_job(_make_async_job(jobs.YumUpdateToPackage,
            desired_pkg, async=async, metadata=metadata))

    @job_request(async=True)
    def update_package(self, pkg,
            async=False,
            to_epoch=None,
            to_version=None,
            to_release=None,
            force=False,
            **metadata):
        """
        @param pkg is an instance of PackageInfo, which must be installed

        The other parameters filter candidate available packages for update.
        """
        return self._do_job(_make_async_job(jobs.YumUpdatePackage,
            pkg, async, to_epoch, to_version, to_release, force=force,
            metadata=metadata))

    @job_request(async=True)
    def check_package(self, pkg, async=False, **metadata):
        """
        Return all necessary information from package database for package
        verification.

        :param pkg: (``PackageInfo``) An instance of PackageInfo
            representing installed package or its nevra string.
        :rtype: (``PackageCheck``)
        """
        return self._do_job(_make_async_job(jobs.YumCheckPackage,
            pkg, async=async, metadata=metadata))

    @job_request(async=True)
    def check_package_file(self, pkg, file_name, async=False):
        """
        Return all necessary information from package database concerning
        on particular file of package. If ``pkg`` does not contain
        ``file_name``, ``FileNotFound`` error is raised.

        :param pkg: (``PackageInfo``) An instance of PackageInfo
            representing installed package or its nevra string.
        :rtype: (``PackageFile``)
        """
        return self._do_job(_make_async_job(jobs.YumCheckPackageFile,
            pkg, file_name, async=async))

    @job_request(async=True)
    def install_package_from_uri(self, uri,
            async=False, update_only=False, force=False, **metadata):
        """
        Install package from uri.
        @param uri is either remote url or local path.
        """
        return self._do_job(_make_async_job(jobs.YumInstallPackageFromURI,
            uri, async, update_only, force=force, metadata=metadata))

    # *************************************************************************
    # Control of asynchronous jobs
    # *************************************************************************
    @job_request()
    def get_job(self, jobid):
        """
        Return instance of ``YumJob`` with given ``jobid``.
        """
        return self._do_job(jobs.YumJobGet(jobid))

    @job_request()
    def get_job_list(self):
        """
        Return list of all asynchronous jobs.
        """
        return self._do_job(jobs.YumJobGetList())

    @job_request()
    def get_job_by_name(self, name):
        """
        Return asynchronous job filtered by its name.
        """
        return self._do_job(jobs.YumJobGetByName(name))

    @job_request()
    def set_job_priority(self, jobid, priority):
        """
        Change priority of asynchronous job. This will change
        its order in queue, if it is still enqeueued.

        Return object of job.
        """
        return self._do_job(jobs.YumJobSetPriority(jobid, priority))

    @job_request()
    def update_job(self, jobid, **kwargs):
        """
        Update metadata of job.

        :param kwargs: (``dict``) Is a dictionary of job's property names
            with mapped new values. Only keys given will be changed in
            desired job.

            **Note** that only keys, that do not affect job's priority or its
            scheduling for deletion can be changed. See :ref:`YumJobUpdate`.
        """
        return self._do_job(jobs.YumJobUpdate(jobid, **kwargs))

    @job_request()
    def reschedule_job(self, jobid,
            delete_on_completion, time_before_removal):
        """
        Change the scheduling of job for deletion.

        :param delete_on_completion: (``bool``) Says, whether the job will
            be scheduled for deletion at ``finished + time_before_removal``
            time.
        :param time_before_removal: (``int``) Number of seconds, after the job
            is finished, it will be kept alive.
        """
        return self._do_job(jobs.YumJobReschedule(jobid,
            delete_on_completion, time_before_removal))

    @job_request()
    def delete_job(self, jobid):
        """
        Delete job object. This can be called only on finished job.
        """
        return self._do_job(jobs.YumJobDelete(jobid))

    @job_request()
    def terminate_job(self, jobid):
        """
        Terminate job. This can be called only on *NEW* job.
        """
        return self._do_job(jobs.YumJobTerminate(jobid))




# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""Python Provider for LMI_SoftwareInstallationService

Instruments the CIM class LMI_SoftwareInstallationService

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import Identity
from lmi.software.core import IdentityResource
from lmi.software.core import InstallationService
from lmi.software.core import Job
from lmi.software.yumdb import YumDB

LOG = cmpi_logging.get_logger(__name__)

class LMI_SoftwareInstallationService(CIMProvider2):
    """Instrument the CIM class LMI_SoftwareInstallationService

    A subclass of service which provides methods to install (or update)
    Software Identities in ManagedElements.

    """

    def __init__(self, _env):
        self.values = InstallationService.Values

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        InstallationService.check_path(env, model.path, "path")

        objpath = InstallationService.get_path(env)

        for key, value in objpath.keybindings.items():
            model[key] = value

        model['Caption'] = 'Software installation service for this system.'
        model['CommunicationStatus'] = self.values. \
                CommunicationStatus.Not_Available
        model['Description'] = \
            'Software installation service using YUM packake manager.'
        model['DetailedStatus'] = self.values.DetailedStatus.Not_Available
        model['EnabledDefault'] = self.values.EnabledDefault.Not_Applicable
        model['EnabledState'] = self.values.EnabledState.Not_Applicable
        model['HealthState'] = self.values.HealthState.OK
        model['InstanceID'] = 'LMI:LMI_SoftwareInstallationService'
        model['OperatingStatus'] = self.values.OperatingStatus.Servicing
        model['OperationalStatus'] = [self.values.OperationalStatus.OK]
        model['PrimaryStatus'] = self.values.PrimaryStatus.OK
        model['RequestedState'] = self.values.RequestedState.Not_Applicable
        model['Started'] = True
        model['TransitioningToState'] = self.values. \
                TransitioningToState.Not_Applicable
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        model.path.update({'CreationClassName': None, 'SystemName': None,
            'Name': None, 'SystemCreationClassName': None})

        objpath = InstallationService.get_path(env)
        for key, value in objpath.keybindings.items():
            model[key] = value
        if not keys_only:
            model = self.get_instance(env, model)
        yield model

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate=None,
                                      param_timeoutperiod=None):
        """Implements LMI_SoftwareInstallationService.RequestStateChange()

        Requests that the state of the element be changed to the value
        specified in the RequestedState parameter. When the requested
        state change takes place, the EnabledState and RequestedState of
        the element will be the same. Invoking the RequestStateChange
        method multiple times could result in earlier requests being
        overwritten or lost.  A return code of 0 shall indicate the state
        change was successfully initiated.  A return code of 3 shall
        indicate that the state transition cannot complete within the
        interval specified by the TimeoutPeriod parameter.  A return code
        of 4096 (0x1000) shall indicate the state change was successfully
        initiated, a ConcreteJob has been created, and its reference
        returned in the output parameter Job. Any other return code
        indicates an error condition.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method RequestStateChange()
            should be invoked.
        param_requestedstate --  The input parameter RequestedState (
                type pywbem.Uint16 self.Values.RequestStateChange.RequestedState)
            The state requested for the element. This information will be
            placed into the RequestedState property of the instance if the
            return code of the RequestStateChange method is 0 ('Completed
            with No Error'), or 4096 (0x1000) ('Job Started'). Refer to
            the description of the EnabledState and RequestedState
            properties for the detailed explanations of the RequestedState
            values.

        param_timeoutperiod --  The input parameter TimeoutPeriod (
                type pywbem.CIMDateTime)
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition.  If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of 'Use Of Timeout
            Parameter Not Supported' shall be returned.


        Returns a two-tuple containing the return value (
                type pywbem.Uint32 self.Values.RequestStateChange)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(
                classname='CIM_ConcreteJob', ...))
            May contain a reference to the ConcreteJob created to track the
            state transition initiated by the method invocation.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        out_params = []
        out_params += [pywbem.CIMParameter('Job', type='reference', value=None)]
        return ( self.values.RequestStateChange.Not_Supported
               , out_params)

    @cmpi_logging.trace_method
    def cim_method_stopservice(self, env, object_name):
        """Implements LMI_SoftwareInstallationService.StopService()

        The StopService method places the Service in the stopped state.
        Note that the function of this method overlaps with the
        RequestedState property. RequestedState was added to the model to
        maintain a record (such as a persisted value) of the last state
        request. Invoking the StopService method should set the
        RequestedState property appropriately. The method returns an
        integer value of 0 if the Service was successfully stopped, 1 if
        the request is not supported, and any other number to indicate an
        error. In a subclass, the set of possible return codes could be
        specified using a ValueMap qualifier on the method. The strings to
        which the ValueMap contents are translated can also be specified
        in the subclass as a Values array qualifier.   Note: The semantics
        of this method overlap with the RequestStateChange method that is
        inherited from EnabledLogicalElement. This method is maintained
        because it has been widely implemented, and its simple "stop"
        semantics are convenient to use.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method StopService()
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        rval = pywbem.Uint32(1) # Not Supported
        return (rval, [])

    @cmpi_logging.trace_method
    def cim_method_installfromuri(self, env, object_name,
                                  param_installoptionsvalues=None,
                                  param_uri=None,
                                  param_installoptions=None,
                                  param_target=None):
        """Implements LMI_SoftwareInstallationService.InstallFromURI()

        Start a job to install software from a specific URI in a
        ManagedElement.  Note that this method is provided to support
        existing, alternative download mechanisms (such as used for
        firmware download). The 'normal' mechanism will be to use the
        InstallFromSoftwareIdentity method. If 0 is returned, the function
        completed successfully and no ConcreteJob instance was required.
        If 4096/0x1000 is returned, a ConcreteJob will be started to to
        perform the install. The Job's reference will be returned in the
        output parameter Job.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method InstallFromURI()
            should be invoked.
        param_installoptionsvalues --  The input parameter \
                InstallOptionsValues (type [unicode,])
            InstallOptionsValues is an array of strings providing
            additionalinformation to InstallOptions for the method to
            install the software. Each entry of this array is related to
            the entry in InstallOptions that is located at the same index
            providing additional information for InstallOptions.  For
            further information on the use of InstallOptionsValues
            parameter, see the description of the InstallOptionsValues
            parameter of the
            InstallationService.InstallFromSoftwareIdentity
            method.

        param_uri --  The input parameter URI (type unicode)
            A URI for the software based on RFC 2079.

        param_installoptions --  The input parameter InstallOptions (
                type [pywbem.Uint16,] self.Values.InstallFromURI.InstallOptions)
            Options to control the install process.  See the InstallOptions
            parameter of the
            InstallationService.InstallFromSoftwareIdentity method
            for the description of these values.

        param_target --  The input parameter Target (type REF (
                pywbem.CIMInstanceName(classname='CIM_ManagedElement', ...))
            The installation target.


        Returns a two-tuple containing the return value (
                type pywbem.Uint32 self.Values.InstallFromURI)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(
                classname='CIM_ConcreteJob', ...))
            Reference to the job (may be null if job completed).


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        InstallationService.check_path(env, object_name, "object_name")
        out_params = [pywbem.CIMParameter('Job', type='reference', value=None)]
        try:
            jobid = InstallationService.install_or_remove_package(
                    env, Job.JOB_METHOD_INSTALL_FROM_URI,
                    param_uri, param_target, None, param_installoptions,
                    param_installoptionsvalues)
            rval = self.values.InstallFromURI. \
                    Method_Parameters_Checked___Job_Started
            out_params[0].value = Job.job2model(jobid,
                    class_name="LMI_SoftwareInstallationJob")
        except InstallationService.InstallationError as exc:
            LOG().error("installation failed: %s", exc.description)
            rval = exc.return_code
        return (rval, out_params)

    @cmpi_logging.trace_method
    def cim_method_checksoftwareidentity(self, env, object_name,
                                         param_source=None,
                                         param_target=None,
                                         param_collection=None):
        """Implements LMI_SoftwareInstallationService.CheckSoftwareIdentity()

        This method allows a client application to determine whether a
        specific SoftwareIdentity can be installed (or updated) on a
        ManagedElement. It also allows other characteristics to be
        determined such as whether install will require a reboot. In
        addition a client can check whether the SoftwareIdentity can be
        added simulataneously to a specified SofwareIndentityCollection. A
        client MAY specify either or both of the Collection and Target
        parameters. The Collection parameter is only supported if
        SoftwareInstallationServiceCapabilities.CanAddToCollection is
        TRUE.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method CheckSoftwareIdentity()
            should be invoked.
        param_source --  The input parameter Source (type REF (
                pywbem.CIMInstanceName(classname='CIM_SoftwareIdentity', ...))
            Reference to the SoftwareIdentity to be checked.

        param_target --  The input parameter Target (type REF (
                pywbem.CIMInstanceName(classname='CIM_ManagedElement', ...))
            Reference to the ManagedElement that the Software Identity is
            going to be installed in (or updated).

        param_collection --  The input parameter Collection (type REF (
                pywbem.CIMInstanceName(classname='CIM_Collection', ...))
            Reference to the Collection to which the Software Identity will
            be added.

        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.CheckSoftwareIdentity)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        InstallCharacteristics -- (type [pywbem.Uint16,]
                self.Values.CheckSoftwareIdentity.InstallCharacteristics)
            The parameter describes the characteristics of the
            installation/update that will take place if the Source
            Software Identity is installed:  Target automatic reset: The
            target element will automatically reset once the installation
            is complete.  System automatic reset: The containing system of
            the target ManagedElement (normally a logical device or the
            system itself) will automatically reset/reboot once the
            installation is complete.  Separate target reset required:
            EnabledLogicalElement.RequestStateChange MUST be used to reset
            the target element after the SoftwareIdentity is installed.
            Separate system reset required:
            EnabledLogicalElement.RequestStateChange MUST be used to
            reset/reboot the containing system of the target
            ManagedElement after the SoftwareIdentity is installed.
            Manual Reboot Required: The system MUST be manually rebooted
            by the user.  No reboot required : No reboot is required after
            installation.  User Intervention Recomended : It is
            recommended that a user confirm installation of this
            SoftwareIdentity. Inappropriate application MAY have serious
            consequences.  MAY be added to specified collection : The
            SoftwareIndentity MAY be added to specified Collection.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE)

    @cmpi_logging.trace_method
    def cim_method_changeaffectedelementsassignedsequence(self,
            env, object_name,
            param_managedelements,
            param_assignedsequence):
        """Implements LMI_SoftwareInstallationService. \
                ChangeAffectedElementsAssignedSequence()

        This method is called to change relative sequence in which order
        the ManagedElements associated to the Service through
        CIM_ServiceAffectsElement association are affected. In the case
        when the Service represents an interface for client to execute
        extrinsic methods and when it is used for grouping of the managed
        elements that could be affected, the ordering represents the
        relevant priority of the affected managed elements with respect to
        each other.  An ordered array of ManagedElement instances is
        passed to this method, where each ManagedElement instance shall be
        already be associated with this Service instance via
        CIM_ServiceAffectsElement association. If one of the
        ManagedElements is not associated to the Service through
        CIM_ServiceAffectsElement association, the implementation shall
        return a value of 2 ("Error Occured").  Upon successful execution
        of this method, if the AssignedSequence parameter is NULL, the
        value of the AssignedSequence property on each instance of
        CIM_ServiceAffectsElement shall be updated such that the values of
        AssignedSequence properties shall be monotonically increasing in
        correlation with the position of the referenced ManagedElement
        instance in the ManagedElements input parameter. That is, the
        first position in the array shall have the lowest value for
        AssignedSequence. The second position shall have the second lowest
        value, and so on. Upon successful execution, if the
        AssignedSequence parameter is not NULL, the value of the
        AssignedSequence property of each instance of
        CIM_ServiceAffectsElement referencing the ManagedElement instance
        in the ManagedElements array shall be assigned the value of the
        corresponding index of the AssignedSequence parameter array. For
        ManagedElements instances which are associated with the Service
        instance via CIM_ServiceAffectsElement and are not present in the
        ManagedElements parameter array, the AssignedSequence property on
        the CIM_ServiceAffects association shall be assigned a value of 0.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method
            ChangeAffectedElementsAssignedSequence() should be invoked.
        param_managedelements --  The input parameter ManagedElements (
                type REF (pywbem.CIMInstanceName(
                    classname='CIM_ManagedElement', ...)) (Required)
            An array of ManagedElements.

        param_assignedsequence --  The input parameter AssignedSequence (
                type [pywbem.Uint16,]) (Required)
            An array of integers representing AssignedSequence for the
            ManagedElement in the corresponding index of the
            ManagedElements parameter.


        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values. \
                    ChangeAffectedElementsAssignedSequence)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(
                classname='CIM_ConcreteJob', ...))
            Reference to the job spawned if the operation continues after
            the method returns. (May be null if the task is completed).


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE)

    @cmpi_logging.trace_method
    def cim_method_installfromsoftwareidentity(
            self, env, object_name,
            param_installoptions=None,
            param_target=None,
            param_collection=None,
            param_source=None,
            param_installoptionsvalues=None):
        """Implements LMI_SoftwareInstallationService. \
                InstallFromSoftwareIdentity()

        Start a job to install or update a SoftwareIdentity (Source) on a
        ManagedElement (Target).  In addition the method can be used to
        add the SoftwareIdentity simulataneously to a specified
        SofwareIndentityCollection. A client MAY specify either or both of
        the Collection and Target parameters. The Collection parameter is
        only supported if InstallationService.CanAddToCollection
        is TRUE.  If 0 is returned, the function completed successfully
        and no ConcreteJob instance was required. If 4096/0x1000 is
        returned, a ConcreteJob will be started to perform the install.
        The Job's reference will be returned in the output parameter Job.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method
                InstallFromSoftwareIdentity()
            should be invoked.
        param_installoptions --  The input parameter InstallOptions (
                type [pywbem.Uint16,] self.Values.InstallFromSoftwareIdentity.\
                        InstallOptions)
            Options to control the install process. Defer target/system
            reset : do not automatically reset the target/system. Force
            installation : Force the installation of the same or an older
            SoftwareIdentity. Install: Perform an installation of this
            software on the managed element. Update: Perform an update of
            this software on the managed element. Repair: Perform a repair
            of the installation of this software on the managed element by
            forcing all the files required for installing the software to
            be reinstalled. Reboot: Reboot or reset the system immediately
            after the install or update of this software, if the install
            or the update requires a reboot or reset. Password: Password
            will be specified as clear text without any encryption for
            performing the install or update. Uninstall: Uninstall the
            software on the managed element. Log: Create a log for the
            install or update of the software. SilentMode: Perform the
            install or update without displaying any user interface.
            AdministrativeMode: Perform the install or update of the
            software in the administrative mode. ScheduleInstallAt:
            Indicates the time at which theinstall or update of the
            software will occur.

        param_target --  The input parameter Target (type REF (
                pywbem.CIMInstanceName(classname='CIM_ManagedElement', ...))
            The installation target. If NULL then the SOftwareIdentity will
            be added to Collection only. The underlying implementation is
            expected to be able to obtain any necessary metadata from the
            Software Identity.

        param_collection --  The input parameter Collection (type REF (
                pywbem.CIMInstanceName(classname='CIM_Collection', ...))
            Reference to the Collection to which the Software Identity
            SHALL be added. If NULL then the SOftware Identity will not be
            added to a Collection.

        param_source --  The input parameter Source (type REF (
                pywbem.CIMInstanceName(classname='CIM_SoftwareIdentity', ...))
            Reference to the source of the install.

        param_installoptionsvalues --  The input parameter
                InstallOptionsValues (type [unicode,])
            InstallOptionsValues is an array of strings providing
            additional information to InstallOptions for the method to
            install the software. Each entry of this array is related to
            the entry in InstallOptions that is located at the same index
            providing additional information for InstallOptions.  If the
            index in InstallOptions has the value "Password " then a value
            at the corresponding index of InstallOptionValues shall not be
            NULL.  If the index in InstallOptions has the value
            "ScheduleInstallAt" then the value at the corresponding index
            of InstallOptionValues shall not be NULL and shall be in the
            datetime type format.  If the index in InstallOptions has the
            value "Log " then a value at the corresponding index of
            InstallOptionValues may be NULL.  If the index in
            InstallOptions has the value "Defer target/system reset",
            "Force installation","Install", "Update", "Repair" or "Reboot"
            then a value at the corresponding index of InstallOptionValues
            shall be NULL.


        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.InstallFromSoftwareIdentity)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(
                classname='CIM_ConcreteJob', ...))
            Reference to the job (may be null if job completed).

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        InstallationService.check_path(env, object_name, "object_name")
        out_params = [pywbem.CIMParameter('Job', type='reference', value=None)]
        try:
            jobid = InstallationService.install_or_remove_package(
                    env, Job.JOB_METHOD_INSTALL_FROM_SOFTWARE_IDENTITY,
                    param_source, param_target, param_collection,
                    param_installoptions,
                    param_installoptionsvalues)
            rval = self.values.InstallFromSoftwareIdentity. \
                    Method_Parameters_Checked___Job_Started
            out_params[0].value = Job.job2model(jobid,
                    class_name="LMI_SoftwareInstallationJob")
        except InstallationService.InstallationError as exc:
            LOG().error("installation failed: %s", exc.description)
            rval = exc.return_code
        return (rval, out_params)

    @cmpi_logging.trace_method
    def cim_method_startservice(self, env, object_name):
        """Implements LMI_SoftwareInstallationService.StartService()

        The StartService method places the Service in the started state.
        Note that the function of this method overlaps with the
        RequestedState property. RequestedState was added to the model to
        maintain a record (such as a persisted value) of the last state
        request. Invoking the StartService method should set the
        RequestedState property appropriately. The method returns an
        integer value of 0 if the Service was successfully started, 1 if
        the request is not supported, and any other number to indicate an
        error. In a subclass, the set of possible return codes could be
        specified using a ValueMap qualifier on the method. The strings to
        which the ValueMap contents are translated can also be specified
        in the subclass as a Values array qualifier.   Note: The semantics
        of this method overlap with the RequestStateChange method that is
        inherited from EnabledLogicalElement. This method is maintained
        because it has been widely implemented, and its simple "start"
        semantics are convenient to use.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method StartService()
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        rval = pywbem.Uint32(1)     # Not Supported
        return (rval, [])

    @cmpi_logging.trace_method
    def cim_method_installfrombytestream(self, env, object_name,
                                         param_installoptionsvalues=None,
                                         param_image=None,
                                         param_installoptions=None,
                                         param_target=None):
        """Implements LMI_SoftwareInstallationService.InstallFromByteStream()

        Start a job to download a series of bytes containing a software
        image to a ManagedElement.  Note that this method is provided to
        support existing, alternative download mechanisms (such as used
        for firmware download). The 'normal' mechanism will be to use the
        InstallFromSoftwareIdentity method.  If 0 is returned, the
        function completed successfully and no ConcreteJob instance was
        required. If 4096/0x1000 is returned, a ConcreteJob will be
        started to to perform the install. The Job's reference will be
        returned in the output parameter Job.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method InstallFromByteStream()
            should be invoked.
        param_installoptionsvalues --  The input parameter InstallOptionsValues (type [unicode,])
            InstallOptionsValues is an array of strings providing
            additional information to InstallOptions for the method to
            install the software. Each entry of this array is related to
            the entry in InstallOptions that is located at the same index
            providing additional information for InstallOptions.   For
            further information on the use of InstallOptionsValues
            parameter, see the description of the InstallOptionsValues
            parameter of the
            InstallationService.InstallFromSoftwareIdentity
            method.

        param_image --  The input parameter Image (type [pywbem.Uint8,])
            A array of bytes containing the install image.

        param_installoptions --  The input parameter InstallOptions (type [pywbem.Uint16,] self.Values.InstallFromByteStream.InstallOptions)
            Options to control the install process.  See the InstallOptions
            parameter of the
            InstallationService.InstallFromSoftwareIdentity method
            for the description of these values.

        param_target --  The input parameter Target (type REF (pywbem.CIMInstanceName(classname='CIM_ManagedElement', ...))
            The installation target.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.InstallFromByteStream)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the job (may be null if job completed).


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        out_params = [pywbem.CIMParameter('Job', type='reference', value=None)]
        return ( self.values.InstallFromByteStream.Not_Supported
               , out_params)

    @cmpi_logging.trace_method
    def cim_method_verifyinstalledidentity(self, env, object_name,
                                           param_source=None,
                                           param_target=None):
        """Implements LMI_SoftwareInstallationService. \
                VerifyInstalledIdentity()

        Start a job to verify installed package represented by
        SoftwareIdentity (source) (Source) on a ManagedElement (Target).
        If 0 is returned, the function completed successfully and no
        ConcreteJob instance was required. If 4096/0x1000 is returned, a
        ConcreteJob will be started to perform the verification. The Job's
        reference will be returned in the output parameter Job. In former
        case, the Failed parameterwill contain all associated file checks,
        that did not pass. In the latter case this property will be NULL.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method VerifyInstalledIdentity()
            should be invoked.
        param_source --  The input parameter Source (
                type REF (pywbem.CIMInstanceName(
                    classname='LMI_SoftwareIdentity', ...))
            Reference to the installed SoftwareIdentity to be verified.

        param_target --  The input parameter Target (
                type REF (pywbem.CIMInstanceName(
                    classname='CIM_ManagedElement', ...))
            Reference to the ManagedElement that the Software Identity is
            installed on.


        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.VerifyInstalledIdentity)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(
                classname='LMI_SoftwareVerificationJob', ...))
            Reference to the job (may be null if job completed).

        Failed -- (type REF (pywbem.CIMInstanceName(
                classname='LMI_SoftwareIdentityFileCheck', ...))
            Array of file checks that did not pass verification. This is
            NULL in case that asynchronous job has been started.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        InstallationService.check_path(env, object_name, "object_name")
        out_params = [pywbem.CIMParameter('Job', type='reference', value=None)]
        try:
            jobid = InstallationService.verify_package(
                    env, Job.JOB_METHOD_VERIFY_INSTALLED_IDENTITY,
                    param_source, param_target)
            rval = self.values.VerifyInstalledIdentity. \
                    Method_Parameters_Checked___Job_Started
            out_params[0].value = Job.job2model(jobid,
                    class_name="LMI_SoftwareVerificationJob")
        except InstallationService.InstallationError as exc:
            LOG().error("failed to launch verification job: %s",
                    exc.description)
            rval = exc.return_code
        return (rval, out_params)

    @cmpi_logging.trace_method
    def cim_method_findidentity(self, env, object_name,
            param_name=None,
            param_epoch=None,
            param_version=None,
            param_release=None,
            param_architecture=None,
            param_allowduplicates=None,
            param_exactmatch=None,
            param_repository=None):
        """Implements LMI_SoftwareInstallationService.FindIdentity()

        Search for installed or available software identity matching
        specified properties. In case "Repository" is given, only
        available packages of this repository will be browsed.
        "AllowDuplicates" causes, that packages of the name <name>.<arch>
        will be listed multiple times if more versions are available.
        Other input parameters with non-NULL values are compared to
        corresponding properties of LMI_SoftwareIdentity instances. 0 is
        returned if any matching package is found, 1 otherwise.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method FindIdentity()
            should be invoked.
        param_name --  The input parameter Name (type unicode)
        param_repository --  The input parameter Repository (
                type REF (pywbem.CIMInstanceName(
                    classname='LMI_SoftwareIdentityResource', ...))
            Allows to specify particular software repository, where the
            search shall take place. If given, only available packages
            will be browsed.

        param_epoch --  The input parameter Epoch (type pywbem.Uint32)
        param_version --  The input parameter Version (type unicode)
        param_architecture --  The input parameter Architecture (type unicode)
        param_allowduplicates --  The input parameter AllowDuplicates (
                type bool)
            Whether the different versions of the same package shall be
            included in result. This defaults to "False".
        param_exactmatch --  The input parameter ExactMatch (type bool)
            Whether to compare "Name" for exact match. If "False", package name
            and its summary string ("Caption") will be searched for occurences
            of "Name" parameter's value. Defaults to "True".
        param_release --  The input parameter Release (type unicode)

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Matches -- (type REF (pywbem.CIMInstanceName(
                classname='LMI_SoftwareIdentity', ...))
            All matching packages found shall be available in this
            parameter.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        InstallationService.check_path(env, object_name, "object_name")
        out_params = [pywbem.CIMParameter(
            'Matches', type='reference', value=None, is_array=True)]
        kwargs = {
                'name'             : param_name,
                'epoch'            : param_epoch,
                'version'          : param_version,
                'release'          : param_release,
                'arch'             : param_architecture,
                'allow_duplicates' : param_allowduplicates,
                'exact_match'      : param_exactmatch,
                'sort'             : True,
        }
        if param_repository is not None:
            repo = IdentityResource.object_path2repo(env, param_repository)
            kwargs['repoid'] = repo.repoid
        pkgs = YumDB.get_instance().filter_packages('all', **kwargs)
        out_params[0].value = [Identity.pkg2model(p) for p in pkgs]
        if len(pkgs) > 0:
            rval = self.values.FindIdentity.Found
        else:
            rval = self.values.FindIdentity.NoMatch
        return (rval, out_params)




# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Tomas Bzatek <tbzatek@redhat.com>
#

from journald_common import JournalBase
import time
import pywbem
import unittest

class TestWriting(JournalBase):
    """
    Class for testing LMI_JournalLogRecord record writing
    """

    def test_create_instance(self):
        """
        Journal: Test new record writeout using CreateInstance()
        """
        new_msg = "== LMI_Journald test message for CreateInstance() mark %d ==" % (time.time() * 1000)
        inst = pywbem.CIMInstance('LMI_JournalLogRecord',
                                  properties = { "CreationClassName": "LMI_JournalLogRecord",
                                                 "LogCreationClassName": "LMI_JournalMessageLog",
                                                 "LogName": "Journal",
                                                 "DataFormat": new_msg })
        iname = self.wbemconnection.CreateInstance(NewInstance = inst)
        new_inst = self.wbemconnection.GetInstance(iname)
        self.assertIsNotNone(iname.keybindings['RecordID'])
        self.assertIsNotNone(iname.keybindings['MessageTimestamp'])
        self.assertIn(new_msg, new_inst['DataFormat'])

if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""Python Provider for LMI_SoftwareInstallationJob and
LMI_SoftwareVerificationJob. They will be referred to as LMI_SoftwareJob,
which is their base class.

Instruments the CIM class LMI_SoftwareJob
"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import Job
from lmi.software.yumdb import errors, YumDB

LOG = cmpi_logging.get_logger(__name__)

class LMI_SoftwareJob(CIMProvider2):
    """Instrument the CIM class LMI_SoftwareJob

    A concrete version of Job. This class represents a generic and
    instantiable unit of work, such as a batch or a print job.

    """

    def __init__ (self, _env):
        self.values = Job.Values

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        job = Job.object_path2job(model.path)
        return Job.job2model(job, keys_only=False, model=model)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model.
        model.path.update({'InstanceID': None})

        ch = env.get_cimom_handle()
        for job in YumDB.get_instance().get_job_list():
            if ch.is_subclass(model.path.namespace,
                    sub=model.path.classname,
                    super=Job.job_class2cim_class_name(job.__class__)):
                yield Job.job2model(job, keys_only=keys_only, model=model)

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        if not modify_existing:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Can not create new instance.")
        return Job.modify_instance(instance)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        with YumDB.get_instance() as ydb:
            job = Job.object_path2job(instance_name)
            try:
                LOG().info('deleting job "%s"' % job)
                ydb.delete_job(job.jobid)
                LOG().info('job "%s" removed' % job)
            except errors.JobNotFound as exc:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        getattr(exc, 'message', str(exc)))
            except errors.JobControlError as exc:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        getattr(exc, 'message', str(exc)))

    @cmpi_logging.trace_method
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate=None,
                                      param_timeoutperiod=None):
        """Implements LMI_SoftwareJob.RequestStateChange()

        Requests that the state of the job be changed to the value
        specified in the RequestedState parameter. Invoking the
        RequestStateChange method multiple times could result in earlier
        requests being overwritten or lost.  If 0 is returned, then the
        task completed successfully. Any other return code indicates an
        error condition.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method RequestStateChange()
            should be invoked.
        param_requestedstate --  The input parameter RequestedState (
                type pywbem.Uint16
                self.Values.RequestStateChange.RequestedState)
            RequestStateChange changes the state of a job. The possible
            values are as follows:  Start (2) changes the state to
            'Running'.  Suspend (3) stops the job temporarily. The
            intention is to subsequently restart the job with 'Start'. It
            might be possible to enter the 'Service' state while
            suspended. (This is job-specific.)  Terminate (4) stops the
            job cleanly, saving data, preserving the state, and shutting
            down all underlying processes in an orderly manner.  Kill (5)
            terminates the job immediately with no requirement to save
            data or preserve the state.  Service (6) puts the job into a
            vendor-specific service state. It might be possible to restart
            the job.

        param_timeoutperiod --  The input parameter TimeoutPeriod (
                type pywbem.CIMDateTime)
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition.  If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of 'Use Of Timeout
            Parameter Not Supported' must be returned.


        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.RequestStateChange)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        job = Job.object_path2job(object_name)
        try:
            if param_requestedstate not in {
                    self.values.RequestStateChange.RequestedState.Terminate }:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Valid RequestedState can by only Terminate (%d)" %
                        self.values.RequestStateChange.RequestedState.Terminate)
            if param_timeoutperiod:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Timeout period is not supported.")
            YumDB.get_instance().terminate_job(job.jobid)
        except errors.JobNotFound as exc:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    getattr(exc, 'message', str(exc)))
        except errors.JobControlError as exc:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    getattr(exc, 'message', str(exc)))
        return (self.values.GetErrors.Success, [])

    @cmpi_logging.trace_method
    def cim_method_geterrors(self, env, object_name):
        """Implements LMI_SoftwareJob.GetErrors()

        If JobState is "Completed" and Operational Status is "Completed"
        then no instance of CIM_Error is returned.  If JobState is
        "Exception" then GetErrors may return intances of CIM_Error
        related to the execution of the procedure or method invoked by the
        job. If Operatational Status is not "OK" or "Completed"then
        GetErrors may return CIM_Error instances related to the running of
        the job.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method GetErrors()
            should be invoked.

        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.GetErrors)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Errors -- (type pywbem.CIMInstance(classname='CIM_Error', ...))
            If the OperationalStatus on the Job is not "OK", then this
            method will return one or more CIM Error instance(s).
            Otherwise, when the Job is "OK", null is returned.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        job = Job.object_path2job(object_name)
        error = Job.job2error(env, job)
        out_params = []
        if error is not None:
            param = pywbem.CIMParameter('Errors', type='instance',
                    is_array=True, array_size=1, value=[error])
            out_params.append(param)
        return (self.values.GetErrors.Success, out_params)

    @cmpi_logging.trace_method
    def cim_method_killjob(self, env, object_name,
                           param_deleteonkill=None):
        """Implements LMI_SoftwareJob.KillJob()

        KillJob is being deprecated because there is no distinction made
        between an orderly shutdown and an immediate kill.
        CIM_ConcreteJob.RequestStateChange() provides 'Terminate' and
        'Kill' options to allow this distinction.  A method to kill this
        job and any underlying processes, and to remove any 'dangling'
        associations.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method KillJob()
            should be invoked.
        param_deleteonkill --  The input parameter DeleteOnKill (type bool)
            Indicates whether or not the Job should be automatically
            deleted upon termination. This parameter takes precedence over
            the property, DeleteOnCompletion.


        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.KillJob)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE)

    @cmpi_logging.trace_method
    def cim_method_geterror(self, env, object_name):
        """Implements LMI_SoftwareJob.GetError()

        GetError is deprecated because Error should be an array,not a
        scalar. When the job is executing or has terminated without error,
        then this method returns no CIM_Error instance. However, if the
        job has failed because of some internal problem or because the job
        has been terminated by a client, then a CIM_Error instance is
        returned.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method GetError()
            should be invoked.

        Returns a two-tuple containing the return value (
            type pywbem.Uint32 self.Values.GetError)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Error -- (type pywbem.CIMInstance(classname='CIM_Error', ...))
            If the OperationalStatus on the Job is not "OK", then this
            method will return a CIM Error instance. Otherwise, when the
            Job is "OK", null is returned.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        job = Job.object_path2job(object_name)
        error = Job.job2error(env, job)
        out_params = []
        if error is not None:
            param = pywbem.CIMParameter('Error', type='instance', value=error)
            out_params.append(param)
        return (self.values.GetErrors.Success, out_params)



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""Python Provider for LMI_SoftwareIdentity

Instruments the CIM class LMI_SoftwareIdentity

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import Identity
from lmi.software.yumdb import YumDB

class LMI_SoftwareIdentity(CIMProvider2):
    """Instrument the CIM class LMI_SoftwareIdentity

    SoftwareIdentity provides descriptive information about a software
    component for asset tracking and/or installation dependency
    management. When the IsEntity property has the value TRUE, the
    instance of SoftwareIdentity represents an individually identifiable
    entity similar to Physical Element. SoftwareIdentity does NOT indicate
    whether the software is installed, executing, etc. This extra
    information may be provided through specialized associations to
    Software Identity. For instance, both InstalledSoftwareIdentity and
    ElementSoftwareIdentity may be used to indicate that the software
    identified by this class is installed. SoftwareIdentity is used when
    managing the software components of a ManagedElement that is the
    management focus. Since software may be acquired, SoftwareIdentity can
    be associated with a Product using the ProductSoftwareComponent
    relationship. The Application Model manages the deployment and
    installation of software via the classes, SoftwareFeatures and
    SoftwareElements. SoftwareFeature and SoftwareElement are used when
    the software component is the management focus. The
    deployment/installation concepts are related to the asset/identity
    one. In fact, a SoftwareIdentity may correspond to a Product, or to
    one or more SoftwareFeatures or SoftwareElements - depending on the
    granularity of these classes and the deployment model. The
    correspondence of Software Identity to Product, SoftwareFeature or
    SoftwareElement is indicated using the ConcreteIdentity association.
    Note that there may not be sufficient detail or instrumentation to
    instantiate ConcreteIdentity. And, if the association is instantiated,
    some duplication of information may result. For example, the Vendor
    described in the instances of Product and SoftwareIdentity MAY be the
    same. However, this is not necessarily true, and it is why vendor and
    similar information are duplicated in this class.  Note that
    ConcreteIdentity can also be used to describe the relationship of the
    software to any LogicalFiles that result from installing it. As above,
    there may not be sufficient detail or instrumentation to instantiate
    this association.

    """

    def __init__(self, _env):
        pass

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        with YumDB.get_instance():
            pkg_info = Identity.object_path2pkg(model.path, 'all')
            return Identity.pkg2model(pkg_info, keys_only=False, model=model)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # too many available packages to enumerate them
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "Enumeration of instances is not supported.")

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)




# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Common utilities and base class for all software tests.
"""

import itertools
import functools
import os
import re
import shutil
import subprocess
import tempfile

from lmi.test.lmibase import LmiTestCase
import package
import repository
import reposetup

RE_REPOPKG = re.compile(r'^(?P<repo>[a-z-]+)#(?P<pkg_name>\w+.*|\*)', re.I)

def test_with_repos(*enable_repos, **repos_dict):
    """
    Decorator factory for test case methods. It enables or disables
    specified repositories for the time of method execution.

    It accepts shortened form of test repositories e.g. *stable* instead of
    *openlmi-sw-test-repo-stable*.

    Usage: ::

        @test_with_repos('stable', **{'updates' : False})
        def test_method(self):
            pass

    Repository names that shall be enabled can be passed as string arguments.
    Those that shall be disabled need to be passed as keyword arguments with
    ``False`` assigned to them.
    """

    def _decorator(func):
        """ Returns wrapped function. """
        @functools.wraps(func)
        def _wrapper(self, *args, **kwargs):
            """ This is a wrapper taking care of repository enablement. """
            to_enable = set(reposetup.full_repo_name(r) for r in enable_repos)
            to_enable.update(
                    set(reposetup.full_repo_name(r)
                for r, v in repos_dict.items() if v))
            to_enable_final = []
            to_disable = []
            for repoid in self.repodb:
                enable = (  repoid in to_enable
                         or bool(repos_dict.get(repoid, False)))
                repo = self.repodb[repoid]
                repo.refresh()
                if enable != repo.status:
                    if enable:
                        to_enable_final.append(repoid)
                    else:
                        to_disable.append(repoid)
            repository.set_repos_enabled(to_enable_final, True)
            repository.set_repos_enabled(to_disable, False)
            for repoid in itertools.chain(to_enable_final, to_disable):
                self.repodb[repoid].refresh()
            return func(self, *args, **kwargs)

        return _wrapper

    return _decorator

def test_with_packages(*install_pkgs, **enable_dict):
    """
    Decorator factory for test case methods. Returned decorator takes care of
    installation or removal of specified packages for the time of method's
    execution.

    Packages needs to be selected from testing database. They are specified in
    form: ::

        repository#package

    Where *repository* marks its full or shortened name. The same applies to
    packages ( *pkg1* instead of *openlmi-sw-test-pkg1* can be written).

    Star ``*`` is also accepted in place of package name marking any package
    present in particular repository.

    When supplied as a key-value pairs, value denote whether package (key)
    shall be installed or removed. Packages to be removed may also omit
    repository part.

    Usage: ::

        @test_with_packages('stable#pkg1', **{
            'updates#pkg2' : True,
            'pkg3' : False
        })
        def test_method(self):
            pass

    Example above shows two possible ways of specifying packages to install.
    Those are:
        * openlmi-sw-test-pkg1 from stable repository
        * openlmi-sw-test-pkg2 from updates repository
    Package openlmi-sw-test-pkg3 will be removed regardless of the source
    repository.
    """

    def _decorator(func):
        @functools.wraps(func)
        def _wrapper(self, *args, **kwargs):
            to_install = []
            to_remove = set()
            for repopkg in itertools.chain(install_pkgs, enable_dict.keys()):
                match = RE_REPOPKG.match(repopkg)
                if match:
                    repo = self.repodb[
                            reposetup.full_repo_name(match.group('repo'))]
                    pkg_name = match.group('pkg_name')
                    if pkg_name == '*':
                        pkgs = list(repo.packages)
                    else:
                        pkgs = [repo[reposetup.full_pkg_name(pkg_name)]]
                    for pkg in pkgs:
                        if not enable_dict.get(repopkg, True):
                            to_remove.add(pkg)
                        else:
                            to_install.append(pkg)
                elif enable_dict.get(repopkg, True) is False:
                    # package is given as string (with just name)
                    to_remove.add(reposetup.full_pkg_name(repopkg))
                    continue
                else:
                    raise ValueError(
                            'invalid format of repo#package string: "%s"'
                            % repopkg)
            to_remove = package.filter_installed_packages(
                    to_remove.union(set(p.name for p in to_install)))
            package.remove_pkgs(to_remove, suppress_stderr=True)
            to_install = package.filter_installed_packages(
                    to_install, installed=False)
            package.install_pkgs(to_install)
            return func(self, *args, **kwargs)

        return _wrapper

    return _decorator

class SwTestCase(LmiTestCase):
    """
    Base class for all LMI Software test classes.

    There are few important class properties to note:

        ``repodb`` : dictionary
            Is a dictionary of testing repositories. These are created upon
            set up of this class. It has following format: ::

                ( repoid, repository)

            Where repository is an instance of
            ;py:class:`repository.Repository`.
        ``other_repos`` : dictionary
            Has the same format but contains other repositories not present in
            ``repodb``. These are disabled upon class's set up and re-enabled
            on its tear down.

    These additional environment variables affects the execution:

        ``LMI_SOFTWARE_YUM_REPOS_DIR``
            Absolute path to directory used to store yum repository
            configuration files.
        ``LMI_SOFTWARE_DB_CACHE``
            File path to a serialized testing database which was cached in
            previous runs.
        ``LMI_SOFTWARE_CLEANUP_CACHE`` : defaults to '1'
            Boolean flag indicating whether, the cache shall be deleted. This
            includes temporary directory with testing repositories and their
            configuration files and serialized database file.
    """
    #: Define in subclass.
    KEYS = tuple()
    #: When running multiple test modules under same process, the test
    #: repository will be cached until last test case is destroyed. This
    #: greatly speeds up the testing.
    TEST_CASES_INSTANTIATED = 0

    def __init__(self, *args, **kwargs):
        LmiTestCase.__init__(self, *args, **kwargs)
        self.longMessage = True

    def get_repo(self, repoid):
        """
        Get repository object from its name. Prefer testin repositories.
        Accepts shortened names of testing repositories.

        :param string repoid: Repository id which may be shortened in case of
            testing one.
        :returns: Repository object.
        :rtype: :py:class:`repository.Repository`
        """
        try:
            return self.repodb[repoid]
        except KeyError:
            try:
                return self.repodb[reposetup.full_repo_name(repoid)]
            except KeyError:
                return self.other_repos[repoid]

    @classmethod
    def setUpClass(cls):
        LmiTestCase.setUpClass.im_func(cls)
        # This applies to all the commands whose output needs to be parsed.
        # Make sure we don't need to deal with localisations.
        os.environ['LANG'] = 'C'
        SwTestCase.yum_repos_dir = os.environ.get(
                'LMI_SOFTWARE_YUM_REPOS_DIR', '/etc/yum.repos.d')
        SwTestCase._cleanup_db = (
                    os.environ.get('LMI_SOFTWARE_CLEANUP_CACHE', '1')
                in  ('1', 'yes', 'on', 'true'))
        if SwTestCase.TEST_CASES_INSTANTIATED == 0:
            if os.environ.get('LMI_SOFTWARE_DB_CACHE', None):
                db_cache = os.environ['LMI_SOFTWARE_DB_CACHE']
                SwTestCase.repos_dir, SwTestCase.repodb, \
                        SwTestCase.other_repos = \
                            reposetup.load_test_database(db_cache)
            if (   not hasattr(SwTestCase, 'repos_dir')
               or not os.path.exists(SwTestCase.repos_dir)
               or any(   not os.path.exists(r.config_path)
                     for r in cls.repodb.values())):
                SwTestCase.repos_dir = tempfile.mkdtemp()
                SwTestCase.repodb, SwTestCase.other_repos = \
                        reposetup.make_object_database(SwTestCase.repos_dir)
            SwTestCase._restore_repos = []
            for repoid in repository.get_repo_list('enabled'):
                if repoid in SwTestCase.other_repos \
                        and repository.is_repo_enabled(repoid):
                    SwTestCase._restore_repos.append(repoid)
            repository.set_repos_enabled(SwTestCase._restore_repos, False)
            for repoid in SwTestCase._restore_repos:
                SwTestCase.other_repos[repoid].refresh()
        SwTestCase.TEST_CASES_INSTANTIATED += 1

    @classmethod
    def tearDownClass(cls):
        if SwTestCase.TEST_CASES_INSTANTIATED <= 1:
            to_uninstall = set()
            for repo in SwTestCase.repodb.values():
                for pkg in repo.packages:
                    to_uninstall.add(pkg.name)
                if SwTestCase._cleanup_db:
                    os.remove(repo.config_path)
            if not SwTestCase._cleanup_db:
                repository.set_repos_enabled(SwTestCase.repodb.keys(), False)
            to_uninstall = package.filter_installed_packages(to_uninstall)
            if to_uninstall:
                package.remove_pkgs(to_uninstall, suppress_stderr=True)
            repository.set_repos_enabled(cls._restore_repos, True)
            if SwTestCase._cleanup_db:
                shutil.rmtree(cls.repos_dir)
                if os.environ.get('LMI_SOFTWARE_DB_CACHE', None):
                    os.remove(os.environ['LMI_SOFTWARE_DB_CACHE'])
            cls.repodb.clear()
        SwTestCase.TEST_CASES_INSTANTIATED -= 1




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""
Just a common functionality related to associated class ``CIM_ComputerSystem``.
"""

import pywbem

from lmi.base.BaseConfiguration import BaseConfiguration
from lmi.providers import cmpi_logging
from lmi.providers import is_this_system

class UninitializedError(Exception):
    """
    Raised when the initialization parameter is not passed to function upon
    its first invocation.
    """
    pass

class NotInstrumentedError(Exception):
    """
    Raised when some CIM class is not instrumented in current *CIMOM*.
    """
    pass

@cmpi_logging.trace_function
def get_path(env=None):
    """
    Get the instance name of preferred computer system. Instance is cached,
    so the broker is accessed just upon the first invocation. Confiruation
    instance must be instantiated before this function is called.

    :param ProviderEnvironment env: Provider environment, taken from CIMOM
        callback (e.g. ``get_providers()``). It's needed just upon the first
        invocation.
    :returns: Instance of ``CIM_ComputerSystem``. Exact subclass is defined
        in configuration file.
    :rtype: :py:class:`pywbem.CIMInstanceName`
    """
    if not hasattr(get_path, '_instance'):
        if env is None:
            raise UninitializedError("get_computer_system_path() needs to be"
                    " given environment object at first invocation")
        inames = [ i for i in env.get_cimom_handle().EnumerateInstanceNames(
                        "root/cimv2",
                        BaseConfiguration.INSTANCE.system_class_name) ]
        if len(inames) < 1:
            raise NotInstrumentedError("CIM_ComputerSystem class is not"
                    " instrumented or EnumerateInstanceNames() operation"
                    " is not supported")
        get_path._instance = inames[0]
    return get_path._instance.copy()

@cmpi_logging.trace_function
def get_system_name(env=None):
    """
    Get the name of system contained in a ``Name`` property of instance of
    ``CIM_ComputerSystem``.

    :param ProviderEnvironment env: Provider environment, taken from CIMOM
        callback (e.g. ``get_providers()``). It's needed only upon a
        first invocation of this function or
        :py:func:`get_computer_system_path`.
    :returns: Name of computer system.
    :rtype: string
    """
    return get_path(env)["Name"]

@cmpi_logging.trace_function
def check_path(env, system, prop_name):
    """
    Checks instance name of ``CIM_ComputerSystem``. If the object path matches
    this computer system, ``True`` is returned. Exception is raised otherwise.
    Especially useful when checking arguments of instance's method taking
    reference to computer system an argument.

    :param system: Instance name of computer system to check.
    :type system: :py:class:`pywbem.CIMInstanceName`
    :param string prop_name: Name of argument or property referring to the object
        path being checked. This is used in exception messages upon
        unsuccessful checks.
    :returns: ``True``
    :rtype: boolean
    """
    if not isinstance(system, pywbem.CIMInstanceName):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "\"%s\" must be a CIMInstanceName" % prop_name)
    our_system = get_path(env)
    ch = env.get_cimom_handle()
    if system.namespace != our_system.namespace:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                'Namespace of "%s" does not match "%s"' % (
                    prop_name, our_system.namespace))
    if not ch.is_subclass(our_system.namespace,
            sub=system.classname,
            super="CIM_ComputerSystem"):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Class of \"%s\" must be a sublass of %s" % (
                    prop_name, our_system.classname))
    if not 'CreationClassName' in system or not 'Name' in system:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "\"%s\" is missing one of keys", prop_name)
    if not ch.is_subclass(our_system.namespace,
            sub=system['CreationClassName'],
            super="CIM_ComputerSystem"):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "CreationClassName of \"%s\" must be a sublass of %s" % (
                prop_name, our_system.classname))
    if not is_this_system(system['Name']):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Name of \"%s\" does not match \"%s\"" % (
                prop_name, our_system['Name']))
    return True

@cmpi_logging.trace_function
def check_path_property(env, op, prop_name):
    """
    This checks, whether the object path contains correct instance name of
    ``CIM_ComputerSystem`` corresponding to this system. If not, an exception
    is raised.

    Example usage in ``get_instance()`` method of provider: ::

        # body of get_instance() method of some association class having
        # *System* property referring to an instance of *CIM_ComputerSystem*.
        ComputerSystem.check_path_property(env, model, "System")

    :param op: Object path to check.
    :param string prop_name: Name of property referring to the object path being
        checked. This is used in exception messages when check is unsuccessful.
    :returns: ``True``
    :rtype: boolean
    """
    if not prop_name in op:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Missing %s key property!" % prop_name)
    return check_path(env, op[prop_name], prop_name)




#!/usr/bin/python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#

import sys
import pywbem

commands = {
    'start': 'StartService',
    'stop': 'StopService',
    'reload': 'ReloadService',
    'restart': 'RestartService',
    'try-restart': 'TryRestartService',
    'condrestart': 'CondRestartService',
    'reload-or-restart': 'ReloadOrRestartService',
    'reload-or-try-restart': 'ReloadOrTryRestartService',
    'enable': 'TurnServiceOn',
    'disable': 'TurnServiceOff'
}

def sanitize(text):
    return text.replace('"', '').replace("'", "")

if len(sys.argv) < 5:
    print """Usage: %s <address> <service> <operation> [username] [password]
    Connect to CIM server at address and execute an operation on the service.

Available states:
    %s

Example: %s https://127.0.0.1:5989 bluetooth start root redhat""" % (sys.argv[0], "\n    ".join(commands.keys()), sys.argv[0])
    sys.exit(1)

url = sys.argv[1]
service = sanitize(sys.argv[2])
operation = sanitize(sys.argv[3])

username = None
password = None
if len(sys.argv) > 4:
    username = sys.argv[4]
if len(sys.argv) > 5:
    password = sys.argv[5]

cliconn = pywbem.WBEMConnection(url, (username, password))
cliconn.debug = True

services = cliconn.ExecQuery("WQL", "select * from LMI_Service where Name='%s'" % service)

if len(services) == 0:
    print >>sys.stderr, "Service %s doesn't exist" % service 
    sys.exit(2)

if len(services) > 1:
    print >>sys.stderr, "There are more services with name '%s', don't know which one use" % service
    sys.exit(3)

if operation not in commands.keys():
    print >>sys.stderr, "Invalid operation '%s'" % operation
    sys.exit(4)

print cliconn.InvokeMethod(commands[operation], services[0].path)



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
"""
Pylint plugin supressing error concerning the use of cmpi_logging
facilities.
"""
from logilab.astng import scoped_nodes
from logilab.astng import MANAGER
from logilab.astng.builder import ASTNGBuilder
from pylint.interfaces import IASTNGChecker
from pylint.checkers import BaseChecker

def logging_transform(module):
    """
    Make the logging.RootLogger look, like it has following methods:
        * trace_warn
        * trace_info
        * trace_verbose
    These are actually defined in its subclass CMPILogger.
    But pylint thinks, that RootLogger is used all the time.
    """
    if module.name != 'logging' or 'RootLogger' not in module:
        return
    fake = ASTNGBuilder(MANAGER).string_build('''

def trace_warn(self, msg, *args, **kwargs):
    print(self, msg, args, kwargs)

def trace_info(self, msg, *args, **kwargs):
    print(self, msg, args, kwargs)

def trace_verbose(self, msg, *args, **kwargs):
    print(self, msg, args, kwargs)

''')
    for level in ('warn', 'info', 'verbose'):
        method_name = 'trace_' + level
        for child in module.locals['RootLogger']:
            if not isinstance(child, scoped_nodes.Class):
                continue
            child.locals[method_name] = fake.locals[method_name]

class CMPILoggingChecker(BaseChecker):
    """
    Checker that only disables some error messages of our custom
    cmpi_logging facilities.
    """

    __implements__ = IASTNGChecker

    name = 'cmpi_logging'
    # When we do have some real warning messages/reports,
    # remote W9921. We need it just to get registered.
    msgs = { 'W9921': ('Dummy', "This is a dummy message.")}
    
    def visit_class(self, node):
        """
        This supresses some warnings for CMPILogger class.
        """
        if (   isinstance(node.parent, scoped_nodes.Module)
           and node.parent.name.split('.')[-1] == "cmpi_logging"
           and node.name == "CMPILogger"):
            # supress warning about too many methods defined
            self.linter.disable('R0904', scope='module', line=node.lineno)

def register(linter):
    """
    Called when loaded by pylint --load-plugins.
    Let's register our tranformation function and checker here.
    """
    MANAGER.register_transformer(logging_transform)
    linter.register_checker(CMPILoggingChecker(linter))



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Module with definition of RPM package check class.
"""

from collections import OrderedDict
from datetime import datetime
import grp
import logging
import os
import pwd
import rpm
import yum

from lmi.providers import cmpi_logging
from lmi.software.yumdb import errors

CHECKSUMTYPE_STR2NUM = dict((val.lower(), k) for (k, val) in
        yum.constants.RPM_CHECKSUM_TYPES.items())

( FILE_TYPE_UNKNOWN
, FILE_TYPE_FILE
, FILE_TYPE_DIRECTORY
, FILE_TYPE_SYMLINK
, FILE_TYPE_FIFO
, FILE_TYPE_CHARACTER_DEVICE
, FILE_TYPE_BLOCK_DEVICE
) = range(7)

FILE_TYPE_NAMES = ( 'unknown', 'file', 'directory', 'symlink', 'fifo'
                  , 'character device', 'block device')

DEFAULT_FILE_DIGEST_ALGO = 'md5'
#: Rpm package header contains information about files encoded in tuples.
#: This is an index to such a tuple pointing to a digest string.
DIGEST_POS_IN_FILE_TUPLE = 12

#: This maps length of digest string encoded in hexadecimal format to
#: corresponding algorithm name.
DIGEST_LENGTH_TO_NAME = {
    32  : 'md5',
    40  : 'sha1',
    64  : 'sha256',
    96  : 'sha384',
    128 : 'sha512',
    56  : 'sha224'
}

LOG = cmpi_logging.get_logger(__name__)

class PackageFile(object):
    """
    Metadata related to particular file on filesystem belonging to RPM package.
    Data contained here are from RPM database.

    Attributes:
      ``path``      - (``str``) Absolute path of file.
      ``file_type`` - (``int``) One of ``FILE_TYPE_*`` identifiers above.
      ``uid``       - (``int``) User ID.
      ``gid``       - (``int``) Group ID.
      ``mode``      - (``int``) Raw file mode.
      ``device``    - (``int``) Raw device number (st_rdev of stat object).
      ``mtime``     - (``int``) Last modification time in seconds.
      ``size``      - (``long``) File size as a number of bytes.
      ``link_target`` - (``str``) Link target of symlink. None if ``file_type``
                        is not symlink.
      ``checksum``  - (``str``) Checksum as string in hexadecimal format.
                       None if file is not a regular file.
    """
    __slots__ = ("path", "file_type", "uid", "gid", "mode", "device", "mtime",
            "size", "link_target", "checksum")

    def __init__(self, path, file_type, uid, gid, mode, device, mtime, size,
            link_target, checksum):
        if not isinstance(file_type, basestring):
            raise TypeError("file_type must be a string")
        for arg in ('uid', 'gid', 'mode', 'mtime', 'size'):
            if not isinstance(locals()[arg], (int, long)):
                raise TypeError("%s must be integer" % arg)
        if not os.path.isabs(path):
            raise ValueError("path must be an absolute path")
        self.path = path
        try:
            self.file_type = FILE_TYPE_NAMES.index(file_type.lower())
        except ValueError:
            logging.getLogger(__name__).error('unrecognized file type "%s" for'
                    ' file "%s"', file_type, path)
            self.file_type = FILE_TYPE_NAMES[FILE_TYPE_UNKNOWN]
        self.uid = uid
        self.gid = gid
        self.mode = mode
        self.device = device
        self.mtime = mtime
        self.size = size
        self.link_target = (link_target
                if self.file_type == FILE_TYPE_SYMLINK else None)
        self.checksum = checksum if self.file_type == FILE_TYPE_FILE else None

    @property
    def last_modification_datetime(self):
        """
        @return instance datetime for last modification time of file
        """
        return datetime.fromtimestamp(self.mtime)

    def __getstate__(self):
        """
        Used for serialization with pickle.
        @return container content that will be serialized
        """
        return dict((k, getattr(self, k)) for k in self.__slots__)

    def __setstate__(self, state):
        """
        Used for deserialization with pickle.
        Restores the object from serialized form.
        @param state is an object created by __setstate__() method
        """
        for k, value in state.items():
            setattr(self, k, value)

class PackageCheck(object):
    """
    Metadata for package concerning verification.
    It contains metadata for each file installed in "files" attribute.
    """
    __slots__ = ("objid", "file_checksum_type", "files")

    def __init__(self, objid, file_checksum_type, files=None):
        """
        @param objid is an in of original yum package object, which is used
        by server for subsequent operations on this package requested by client
        """
        if files is not None and not isinstance(
                files, (list, tuple, set, dict)):
            raise TypeError("files must be an iterable container")
        self.objid = objid
        self.file_checksum_type = file_checksum_type
        if not isinstance(files, dict):
            self.files = OrderedDict()
            if files is not None:
                for file_check in sorted(files, key=lambda f: f.path):
                    self.files[file_check.path] = file_check
        else:
            for path in sorted(files):
                self.files[path] = files[path]

    def __iter__(self):
        return iter(self.files)

    def __len__(self):
        return len(self.files)

    def __getitem__(self, filepath):
        return self.files[filepath]

    def __setitem__(self, filepath, package_file):
        if not isinstance(package_file, PackageFile):
            raise TypeError("package_file must be a PackageFile instance")
        self.files[filepath] = package_file

    def __contains__(self, fileobj):
        if isinstance(fileobj, basestring):
            return fileobj in self.files
        elif isinstance(fileobj, PackageFile):
            return fileobj.path in self.files
        else:
            raise TypeError("expected file path for argument")

    def __getstate__(self):
        """
        Used for serialization with pickle.
        @return container content that will be serialized
        """
        return dict((k, getattr(self, k)) for k in self.__slots__)

    def __setstate__(self, state):
        """
        Used for deserialization with pickle.
        Restores the object from serialized form.
        @param state is an object created by __setstate__() method
        """
        for k, value in state.items():
            setattr(self, k, value)

def pkg_checksum_type(pkg):
    """
    This returns digest algorithm code used to generate file digests stored in
    rpm package. It is not a constant number for whole distribution. It may
    vary package to package

    There are several ways how to get checksum type out of installed package.

        1. Usually it's obtainable from package header under
           ``RPMTAG_FILEDIGESTALGO`` but sometimes it may not be set.
        2. In that case it can be deduced from digest length of any file
           present in package.
        3. If no regular file is present. Try to query package's ``yumdb_info``
            attribute which contain preferred digest algorithm used in this
            distribution.
        4. Unfortunately this ``yumdb_info`` attribute is present just
           occasionally. In case it's not set, fallback to default digest
           algorithm which is 'md5'.

    I just love yum API -- just kidding.

    ..
        I'd love to point to some documentation, where this is explained. If
        only such documentation existed.

        Find out how to ensure that ``yumdb_info`` is set without running
        package verification.

    :returns: Code of digest algorithm used for package files.
    :rtype: int
    """
    if not isinstance(pkg, yum.packages.PackageObject):
        raise TypeError("pkg must be an instance of YumAvailablePackage")
    if isinstance(pkg, yum.rpmsack.RPMInstalledPackage):
        # in certain cases this may be None
        checksum = pkg.hdr[rpm.RPMTAG_FILEDIGESTALGO]
        if checksum is not None:
            return checksum
    if hasattr(pkg, 'hdr'):
        for filetuple in pkg.hdr.fiFromHeader():
            digest = filetuple[DIGEST_POS_IN_FILE_TUPLE]
            if not digest:
                continue    # not a regular file
            length = len(digest)
            if not length in DIGEST_LENGTH_TO_NAME:
                LOG().warn('unexpected length of digest: %d not in {%s}',
                        length, ", ".join(DIGEST_LENGTH_TO_NAME.keys()))
                continue
            return CHECKSUMTYPE_STR2NUM[DIGEST_LENGTH_TO_NAME[length]]
    LOG().warn('no digest algorithm found in header of package "%s"', pkg.nevra)
    cst_str = getattr(pkg.yumdb_info, 'checksum_type', '').lower()
    return CHECKSUMTYPE_STR2NUM.get(cst_str, CHECKSUMTYPE_STR2NUM['md5'])

def make_package_check_from_db(vpkg, file_name=None):
    """
    Create instance of PackageCheck from instance of
    yum.packages._RPMVerifyPackage.

    :param file_name: (``str``) If not None, causes result to have just
        one instance of ``PackageFile`` matching this file_name.
        If it's not found in the package, ``FileNotFound`` will be raised.
    :rtype (``PackageCheck``)
    """
    if not isinstance(vpkg, yum.packages._RPMVerifyPackage):
        raise TypeError("vpkg must be instance of"
                " yum.packages._RPMVerifyPackage")
    pkg = vpkg.po

    res = PackageCheck(id(pkg), pkg_checksum_type(pkg))
    files = res.files
    for vpf in vpkg:
        if file_name is not None and file_name != vpf.filename:
            continue
        files[vpf.filename] = PackageFile(
            vpf.filename,
            vpf.ftype,
            pwd.getpwnam(vpf.user).pw_uid,
            grp.getgrnam(vpf.group).gr_gid,
            vpf.mode,
            vpf.dev,
            vpf.mtime,
            vpf.size,
            vpf.readlink,
            vpf.digest[1]
        )
        if file_name is not None:
            break
    if file_name is not None and len(files) < 1:
        raise errors.FileNotFound('File "%s" not found in package "%s".' % (
            file_name, pkg.nevra))
    return res




# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

from common import AccountBase
from methods import *
import pywbem

class TestMemberOfGroup(AccountBase):
    """
    Class for testing LMI_MemberOfGroup class
    """
    def test_add_user_to_group(self):
        """
        Account: Test to add user to group
        """
        create_account(self.user_name)
        create_group(self.group_name)
        user = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Account where Name = "%s"' %self.user_name)[0]
        group = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Group where Name = "%s"' %self.group_name)[0]
        tocreate = pywbem.CIMInstance('LMI_MemberOfGroup',
            {'Collection' : group.path,
            'Member' : user.path})
        self.wbemconnection.CreateInstance(tocreate)
        clean_account(self.user_name)
        clean_group(self.group_name)

    def test_remove_user_from_group(self):
        """
        Account: Test remove user from group
        """
        # make sure the account will exist
        create_account(self.user_name)
        i = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Account where Name = "%s"' % self.user_name)[0]
        self.wbemconnection.DeleteInstance(i.path)
        # check if it was really deleted
        self.assertFalse(user_exists(self.user_name))
        clean_account(self.user_name)

    def test_user_in_groups(self):
        """
        Account: Test correct list of groups for user
        """
        create_account(self.user_name)
        create_group(self.group_name)
        add_user_to_group(self.user_name, self.group_name)
        user = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Account where Name = "%s"' %self.user_name)[0]
        ident = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Identity where InstanceID = "LMI:UID:%s"' % user["UserID"])[0]
        insts = self.wbemconnection.Associators(ident.path, AssocClass = 'LMI_MemberOfGroup')
        self.assertEqual(len(insts), 2)
        found_user = False
        found_group = False
        for inst in insts:
            if inst["Name"] == self.user_name:
                found_user = True
            elif inst["Name"] == self.group_name:
                found_group = True
        self.assertTrue(found_user)
        self.assertTrue(found_group)
        clean_account(self.user_name)
        clean_group(self.group_name)



# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
LMI test utilities.
"""

import os
import pywbem
import socket
import unittest

def is_this_system(system_name):
    """
    :returns: Whether the given *system_name* matches the hostname of currently
    running system.
    :rtype: boolean
    """
    return (  socket.gethostbyaddr(system_name)[0]
           == socket.gethostbyaddr(socket.gethostname())[0])

def get_environvar(variable, default='', convert=str):
    """
    Get the value of environment variable.

    :param string variable: Name of environment variable.
    :param default: Any value that should be returned when the variable is not
        set. If None, the conversion won't be done.
    :param callable convert: Function transforming value to something else.
    :returns: Converted value of the environment variable.
    """
    val = os.environ.get(variable, default)
    if convert is bool:
        return val.lower() in ('true', 'yes', 'on', '1')
    if val is None:
        return None
    return convert(val)

def mark_dangerous(method):
    """
    Decorator for methods of :py:class:`unittest.TestCase` subclasses that
    skips dangerous tests if an environment variable says so.
    ``LMI_RUN_DANGEROUS`` is the environment variabled read.

    These tests will be skipped by default.
    """
    if get_environvar('LMI_RUN_DANGEROUS', '0', bool):
        return method
    else:
        return unittest.skip("This test is marked as dangerous.")(method)

def mark_tedious(method):
    """
    Decorator for methods of :py:class:`unittest.TestCase` subclasses that
    skips tedious tests. Those running for very long time and usually need a
    lot of memory. They are run by default. Environment variable
    ``LMI_RUN_TEDIOUS`` can be used to skip them.
    """
    if get_environvar('LMI_RUN_TEDIOUS', '1', bool):
        return method
    else:
        return unittest.skip("This test is marked as tedious.")(method)

def check_inames_equal(fst, snd):
    """
    Compare two objects of :py:class:`pywbem.CIMInstanceName`. Their ``host``
    property is not checked. Be benevolent when checking names system
    creation class names.

    :returns: ``True`` if both instance names are equal.
    :rtype: boolean
    """
    if not isinstance(fst, pywbem.CIMInstanceName):
        raise TypeError("fst argument must be a pywbem.CIMInstanceName, not %s"
                % repr(fst))
    if not isinstance(snd, pywbem.CIMInstanceName):
        raise TypeError("snd argument must be a pywbem.CIMInstanceName, not %s"
                % repr(snd))
    if fst.classname != snd.classname or fst.namespace != snd.namespace:
        return False

    snd_keys = { k: v for k, v in snd.keybindings.iteritems() }
    for key, value in fst.keybindings.iteritems():
        if key not in snd_keys:
            return False
        snd_value = snd_keys.pop(key)
        if (   isinstance(value, pywbem.CIMInstanceName)
           and isinstance(snd_value, pywbem.CIMInstanceName)):
            if not check_inames_equal(value, snd_value):
                return False

        # accept also aliases in the Name attribute of ComputerSystem
        elif (  (   fst.classname.endswith('_ComputerSystem')
                and key.lower() == 'name')
             or (   key.lower() == 'systemname'
                and 'SystemCreationClassName' in fst)):
            if (  value != snd_value
               and (  not is_this_system(value)
                   or not is_this_system(snd_value))):
                return False

        elif (   fst.classname.endswith('_ComputerSystem')
             and key.lower() == 'creationclassname'):
            if (   value != snd_value
               and 'CIM_ComputerSystem' not in [
                   p['CreationClassName'] for p in (fst, snd)]):
                return False

        elif value != snd_value:
            return False

    if snd_keys:    # second path has more key properties than first one
        return False

    return True




#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details. #
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
"""
Abstraction for RPM package for test purposes.
"""

import re
import subprocess

import util

RE_NOT_INSTALLED = re.compile(r'package\s+([^[:space:]]+)\s+is not installed',
        re.IGNORECASE)

class Package(object):
    """
    Element of test package database. It's a container for package
    informations.
    """
    def __init__(self, name, epoch, ver, rel, arch, repoid,
            rpm_path, files=None):
        self._name = name
        if not epoch or (isinstance(epoch, basestring)
                and epoch.lower() == "(none)"):
            epoch = "0"
        self._epoch = int(epoch)
        self._ver = ver
        self._rel = rel
        self._arch = arch
        self._repoid = repoid
        self._rpm_path = rpm_path
        if files is None:
            files = set()
        else:
            files = set(files)
        self._files = files

    def __str__(self):
        return self.get_nevra()

    @property
    def name(self):
        """ :returns: Package name. """
        return self._name
    @property
    def epoch(self):
        """ :returns: Package epoch as an integer. """
        return self._epoch
    @property
    def ver(self):
        """ :returns: Version string of package. """
        return self._ver
    @property
    def rel(self):
        """ :returns: Release string of package. """
        return self._rel
    @property
    def arch(self):
        """ :returns: Architecture string of package. """
        return self._arch
    @property
    def repoid(self):
        """ :returns: Repository id of package, where it is available. """
        return self._repoid
    @property
    def nevra(self):
        """
        :returns: Nevra string of package with epoch part always present.
        """
        return self.get_nevra('ALWAYS')
    @property
    def evra(self):
        """
        :returns: Evra string of package. That's the same as *nevra* without a
            name.
        """
        attrs = ('epoch', 'ver', 'rel', 'arch')
        return util.make_evra(*[getattr(self, '_'+a) for a in attrs])
    @property
    def rpm_path(self):
        """
        :returns: Absolute path to rpm package.
        """
        return self._rpm_path

    @property
    def summary(self):
        """
        :returns: Package summary string.
        """
        return subprocess.check_output(
                ['/usr/bin/rpm', '-q', '--qf', '%{SUMMARY}', '-p',
                    self.rpm_path])

    def get_nevra(self, with_epoch='NOT_ZERO'):
        """
        :param string with_epoch: Says when the epoch part should appear
            in resulting string. There are following possible values:

            * NOT_ZERO - epoch shall be present if it's greater than 0
            * ALWAYS - epoch will be present
            * NEVER - epoch won't be present

        :returns: Package nevra string.
        :rtype: string
        """
        attrs = ('name', 'epoch', 'ver', 'rel', 'arch')
        return util.make_nevra(*[getattr(self, '_'+a) for a in attrs],
                with_epoch=with_epoch)

    def __contains__(self, path):
        return path in self._files
    def __iter__(self):
        for pkg in self._files.__iter__():
            yield pkg
    def __len__(self):
        return len(self._files)
    @property
    def files(self):
        """
        :returns: Set of files and directories installed by this package.
        :rtype: set
        """
        return self._files.copy()

def to_json(_encoder, pkg):
    """
    Converts package object to dictionary which json encoder can handle.

    :param _encoder: Instance of json encoder.
    :param pkg: Package object to convert.
    :type pkg: :py:class:`Package`
    :returns: Dictionary with package attributes.
    :rtype: dictionary
    """
    if not isinstance(pkg, Package):
        raise TypeError("not a Package object")
    return pkg.__dict__

def from_json(json_object):
    """
    Constructs a package object from dictionary loaded from json text.
    Inverse function to :py:func:`to_json`.

    :param dictionary json_object: Deserialized package as a dictionary.
    :returns: Package object.
    :rtype: :py:class:`Package`
    """
    if isinstance(json_object, dict) and '_arch' in json_object:
        kwargs = {k[1:]: v for k, v in json_object.items()}
        return Package(**kwargs)
    return json_object

def is_pkg_installed(pkg):
    """
    Check, whether package is installed.
    """
    if not isinstance(pkg, Package):
        return subprocess.call(["rpm", "--quiet", "-q", pkg]) == 0
    else:
        cmd = ["/usr/bin/rpm", "-q", "--qf", "%{EPOCH}:%{NVRA}\n", pkg.nevra]
        try:
            out = subprocess.check_output(cmd).splitlines()[0]
            epoch, _ = out.split(':')
            if not epoch or epoch.lower() == "(none)":
                epoch = "0"
            return int(epoch) == int(pkg.epoch)
        except subprocess.CalledProcessError:
            return False

def filter_installed_packages(pkgs, installed=True):
    """
    Filter the package set returning only installed or not installed packages.

    :param list pkgs: Packages to filter. If removed packages are requested
        only Package objects may be present, otherwise package names or nevra
        strings are allowed.
    :param boolean installed: Whether to return installed or uninstalled
        packages.
    :returns: Filtered packages. Nevra strings are returned for any installed
        packages from *pkgs* represented by strings (either by name or nevra),
        package objects will be returned as package objects.
    :rtype: set
    """
    if len(pkgs) < 1:
        return set()
    pkg_strings = []
    pkg_map = {}
    for pkg in pkgs:
        if isinstance(pkg, Package):
            pkg_map[pkg.nevra] = pkg
            pkg = pkg.nevra
        elif not installed:
            raise TypeError("packages must be objects of Package")
        pkg_strings.append(pkg)
    cmd = [ "/usr/bin/rpm", "-q", "--qf"
          , "%{NAME}-%{EPOCH}:%{VERSION}-%{RELEASE}.%{ARCH}\n"]
    cmd.extend(pkg_strings)
    process = subprocess.Popen(cmd,
            stdout=subprocess.PIPE, stderr=util.DEV_NULL)
    out, _ = process.communicate()
    if installed:
        result = set()
        for line in out.splitlines():
            if not util.RE_NEVRA.match(line):
                continue
            if line in result:
                continue
            result.add(pkg_map.get(line, line))
    else:
        result = set(pkg_strings)
        for match in RE_NOT_INSTALLED.finditer(out):
            if match.group(1) not in pkg_strings:
                raise ValueError('got unexpected package nevra "%s" which'
                        ' should be one of: %s'
                        % (match.group(1), str(pkg_strings)))
            pkg_strings.remove(match.group(1))
            result.remove(result)
        result = {pkg_map.get(pstr, pstr) for pstr in result}
    return result

def remove_pkgs(pkgs, *args, **kwargs):
    """
    Remove package with rpm command.

    :param pkgs: Either an instance of :py:class:`Package` or package name.
        If it's a name, any version will be removed. Otherwise the exact
        version must be installed for command to be successful.
    :param list args: List of parameters for rpm command.
    :param boolean suppress_stderr: Whether the standard error output of ``rpm``
        command shall be suppressed. Defaults to ``False``.
    """
    suppress_stderr = kwargs.pop('suppress_stderr', False)
    cmd = ["rpm", "--quiet"] + list(args) + ['-e']
    if isinstance(pkgs, (basestring, Package)):
        pkgs = [pkgs]
    pkg_strings = []
    for pkg in pkgs:
        if isinstance(pkg, Package):
            pkg_strings.append(pkg.nevra)
        else:
            pkg_strings.append(pkg)
    if len(pkg_strings) > 0:
        cmd.extend(pkg_strings)
        kwargs = {}
        if suppress_stderr:
            kwargs['stderr'] = util.DEV_NULL
        subprocess.call(cmd, **kwargs)

def install_pkgs(pkgs, *args):
    """
    Install a specific package.

    :param pkgs: Package object.
    :type pkgs: :py:class:`Package`
    """
    cmd = ["rpm", "--quiet"] + list(args) + ["-i"]
    if isinstance(pkgs, Package):
        pkgs = [pkgs]
    pkg_paths = []
    for pkg in pkgs:
        if not isinstance(pkg, Package):
            raise TypeError("pkg must be a Package instance")
        pkg_paths.append(pkg.rpm_path)
    if len(pkg_paths) > 0:
        cmd.extend(pkg_paths)
        subprocess.call(cmd)

